#[non_exhaustive]pub enum PackageCacheError {
Cancelled,
LayerError(Box<dyn Error + Send + Sync>),
NoWritableLayers,
UnsafeCacheKey(InvalidPathComponentError),
}Expand description
Errors specific to the PackageCache interface
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Cancelled
The operation was cancelled
LayerError(Box<dyn Error + Send + Sync>)
An error occurred in a cache layer
NoWritableLayers
There are no eligible writable layers to cache the package to.
UnsafeCacheKey(InvalidPathComponentError)
The cache key contains metadata that could lead to path traversal.